@@ -178,7 +178,7 @@ $::application hosts set of Prima::Clipboard objects, created
 automatically to reflect the system-provided clipboard IPC
 functionality. Their number depends on the system, - under X11
 environment there is three clipboard objects, and only one 
-under Win32 and OS/2.
+under Win32.
 
 These are no methods to access these clipboard objects, except
 fetch() ( or, the indirect name calling ) - the clipboard objects
@@ -367,6 +367,24 @@ Default value: 0
 
 =over
 
+=item CopyImage $CLIPBOARD, $IMAGE
+
+The notification stores C<$IMAGE> in clipboard.
+
+=item CopyText $CLIPBOARD, $TEXT
+
+The notification stores C<$TEXT> in clipboard.
+
+=item PasteImage $CLIPBOARD, $$IMAGE_REF
+
+The notification queries C<$CLIPBOARD> for image content and stores in
+C<$$IMAGE_REF>. Default action is that C<'Image'> format is queried.
+On unix, encoded formats C<'image/bmp'>, C<'image/png'> etc are queried if
+the default C<'Image'> is not found.
+
+The C<PasteImage> mechanism is devised to read images from clipboard in 
+GTK environment.
+
 =item PasteText $CLIPBOARD, $$TEXT_REF
 
 The notification queries C<$CLIPBOARD> for text content and stores in
@@ -532,6 +550,11 @@ if 0, the 'lowest' one ( in a simple case when window A is made modal
 If a chain is empty C<undef> is returned. In case when a chain consists
 of just one window, TOPMOST value is apparently irrelevant.
 
+=item get_monitor_rects
+
+Returns set of rects in format [X,Y,WIDTH,HEIGHT] identifying monitor
+configurations. Currently works under X11 only.
+
 =item get_scroll_rate
 
 Returns two integer values of two system-specific
@@ -556,7 +579,6 @@ The hash result contains the following keys:
 One of C<apc::XXX> constants, reflecting the platform.
 Currently, the list of the supported platforms is:
 
-	apc::Os2    
 	apc::Win32  
 	apc::Unix   
 
@@ -642,6 +664,7 @@ of C<sv::XXX> constants. The constants are:
 	sv::ColorPointer     - 1 if system accepts color pointer icons.
 	sv::CanUTF8_Input    - 1 if system can generate key codes in unicode 
 	sv::CanUTF8_Output   - 1 if system can output utf8 text
+	sv::CompositeDisplay - 1 if system uses double-buffering for the desktop
 
 The method can be called with a class string instead of an object instance.
 
