@@ -30,6 +30,7 @@ use strict;
 use Prima::Const;
 use Prima::Classes;
 use Prima::IntUtils;
+use Prima::ScrollWidget;
 
 package Prima::TabSet;
 use vars qw(@ISA);
@@ -148,33 +149,17 @@ sub recalc_widths
 	$self-> {widths}    = [@w];
 }
 
-sub on_mousedown
+sub x2pos
 {
-	my ( $self, $btn, $mod, $x, $y) = @_;
-	return if $self-> {mouseTransaction};
-	$self-> clear_event;
+	my ( $self, $x ) = @_;
+
 	my ( $a, $ww, $ft, $lt) = (
 		$self-> {arrows}, $self-> {widths}, $self-> {firstTab}, $self-> {lastTab}
 	);
-
-	if (( $a & 1) and ( $x < DefLeftX + DefGapX * 2 + DefArrowX)) {
-		$self-> firstTab( $self-> firstTab - 1);
-		$self-> capture(1);
-		$self-> {mouseTransaction} = -1;
-		$self-> scroll_timer_start;
-		$self-> scroll_timer_semaphore(0);
-		return;
-	}
+	return -1 if ( $a & 1) and ( $x < DefLeftX + DefGapX * 2 + DefArrowX);
 
 	my @size = $self-> size;
-	if (( $a & 2) and ( $x >= $size[0] - DefLeftX - DefGapX * 2 - DefArrowX)) {
-		$self-> firstTab( $self-> firstTab + 1);
-		$self-> capture(1);
-		$self-> {mouseTransaction} = 1;
-		$self-> scroll_timer_start;
-		$self-> scroll_timer_semaphore(0);
-		return;
-	}
+	return -2 if ( $a & 2) and ( $x >= $size[0] - DefLeftX - DefGapX * 2 - DefArrowX);
 
 	my $w = DefLeftX;
 	$w += DefGapX + DefArrowX if $a & 1;
@@ -184,13 +169,41 @@ sub on_mousedown
 		$found = $i, last if $x < $w + $$ww[$i] + DefGapX;
 		$w += $$ww[$i] + DefGapX;
 	}
-	return unless defined $found;
-	
-	if ( $found == $self-> {tabIndex}) {
-		$self-> focusedTab( $found);
+	return $found;
+}
+
+sub on_mousedown
+{
+	my ( $self, $btn, $mod, $x, $y) = @_;
+	return if $self-> {mouseTransaction};
+	$self-> clear_event;
+
+	my $pos = $self-> x2pos($x);
+	return unless defined $pos;
+
+	if ($pos == -1) {
+		$self-> firstTab( $self-> firstTab - 1);
+		$self-> capture(1);
+		$self-> {mouseTransaction} = -1;
+		$self-> scroll_timer_start;
+		$self-> scroll_timer_semaphore(0);
+		return;
+	}
+
+	if ($pos == -2) {
+		$self-> firstTab( $self-> firstTab + 1);
+		$self-> capture(1);
+		$self-> {mouseTransaction} = 1;
+		$self-> scroll_timer_start;
+		$self-> scroll_timer_semaphore(0);
+		return;
+	}
+
+	if ( $pos == $self-> {tabIndex}) {
+		$self-> focusedTab( $pos);
 		$self-> focus;
 	} else {
-		$self-> tabIndex( $found);
+		$self-> tabIndex( $pos);
 	}
 }
 
@@ -215,7 +228,16 @@ sub on_mouseup
 sub on_mousemove
 {
 	my ( $self, $mod, $x, $y) = @_;
-	return unless $self-> {mouseTransaction};
+	unless ($self-> {mouseTransaction}) {
+		if ( $self-> enabled ) {
+			my $prelight = $self-> x2pos($x);
+			if (( $prelight // '') ne ($self->{prelight} // '')) {
+				$self->{prelight} = $prelight;
+				$self-> repaint;
+			}
+		}
+		return;
+	}
 	return unless $self-> scroll_timer_semaphore;
 
 	$self-> scroll_timer_semaphore(0);
@@ -224,6 +246,12 @@ sub on_mousemove
 	$self-> notify(q(MouseUp),1,0,0,0) if $ft == $self-> firstTab;
 }
 
+sub on_mouseleave
+{
+	my $self = shift;
+	$self-> repaint if defined( delete $self->{prelight} );
+}
+
 sub on_mouseclick
 {
 	my $self = shift;
@@ -264,8 +292,12 @@ sub on_keydown
 sub on_paint
 {
 	my ($self,$canvas) = @_;
-	my @clr  = ( $self-> color, $self-> backColor);
-	@clr = ( $self-> disabledColor, $self-> disabledBackColor) if ( !$self-> enabled);
+	my @clr;
+	if ( $self-> enabled ) {
+		@clr = ( $self-> color, $self-> backColor);
+	} else {
+		@clr = ( $self-> disabledColor, $self-> disabledBackColor);
+	}
 	my @c3d  = ( $self-> dark3DColor, $self-> light3DColor);
 	my @size = $canvas-> size;
 
@@ -420,10 +452,11 @@ sub on_measuretab
 sub on_drawtab
 {
 	my ( $self, $canvas, $i, $clr, $poly, $poly2) = @_;
-	
-	$canvas-> color(( $self-> {colored} && ( $i >= 0)) ?
-		( $warpColors[ $i % scalar @warpColors]) : $$clr[1]);
-		
+
+	my $color = ( $self-> {colored} && ( $i >= 0)) ?
+		( $warpColors[ $i % scalar @warpColors]) : $$clr[1];
+	$color = $self-> prelight_color($color) if ($self->{prelight} // '') eq ($i // '');
+	$canvas-> color($color);
 	$canvas-> fillpoly( $poly);
 	$canvas-> fillpoly( $poly2) if $poly2;
 	$canvas-> color( $$clr[3]);
@@ -438,7 +471,7 @@ sub on_drawtab
 			$$poly[0] + ( $$poly[6] - $$poly[0] - $self-> {widths}-> [$i]) / 2 + DefGapX * 2,
 			$$poly[1] + ( $$poly[3] - $$poly[1] - $canvas-> font-> height) / 2
 		);
-		$canvas-> text_out( $self-> {tabs}-> [$i], @tx);
+		$canvas-> text_out_bidi( $self-> {tabs}-> [$i], @tx);
 		$canvas-> rect_focus( $tx[0] - 1, $tx[1] - 1,
 			$tx[0] + $self-> {widths}-> [$i] - DefGapX * 4 + 1, $tx[1] + $canvas-> font-> height + 1)
 				if ( $i == $self-> {focusedTab}) && $self-> focused;
@@ -589,6 +622,51 @@ sub set_tabs
 	$self-> update_view;
 }
 
+sub insert_tab
+{
+	my ( $self, $text, $at ) = @_;
+
+	$at = -1 unless defined $at;
+
+	my $t = $self->{tabs};
+	$at = @$t - $at + 1 if $at < 0;
+	return if $at > @$t || $at < 0;
+	splice( @$t, $at, 0, $text );
+
+	my $iw = 0;
+	my ( $notifier, @notifyParms) = $self-> get_notify_sub(q(MeasureTab));
+	$self-> begin_paint_info;
+	$self-> push_event;
+	$notifier-> ( @notifyParms, $at, \$iw);
+	$self-> pop_event;
+	$self-> end_paint_info;
+
+	splice( @{$self->{widths}}, $at, 0,  $iw);
+
+	$self-> reset;
+	$self-> tabIndex( $self-> tabIndex);
+	$self-> repaint;
+
+	return $at;
+}
+
+sub delete_tab
+{
+	my ( $self, $at ) = @_;
+	my $t = $self->{tabs};
+	$at = @$t - $at if $at < 0;
+	return if $at > $#$t || $at < 0;
+	splice( @$t, $at, 1 );
+	splice( @{$self->{widths}}, $at, 1 );
+
+	$self-> reset;
+	$self-> lock;
+	$self-> firstTab( $self-> firstTab);
+	$self-> tabIndex( $self-> tabIndex);
+	$self-> unlock;
+	$self-> update_view;
+}
+
 sub set_top_most
 {
 	my ( $self, $tm) = @_;
@@ -693,21 +771,40 @@ sub insert_page
 	splice( @{$self-> {widgets}}, $at, 0, []);
 	$self-> {pageCount}++;
 	$self-> pageIndex(0) if $self-> {pageCount} == 1;
+
+	return $at;
 }
 
 sub delete_page
 {
 	my ( $self, $at, $removeChildren) = @_;
+
+	return unless $self->{pageCount};
 	
 	$removeChildren = 1 unless defined $removeChildren;
 	$at = -1 unless defined $at;
 	$at = $self-> {pageCount} - 1 if $at < 0 || $at >= $self-> {pageCount};
+
+	my $idx = $self->pageIndex;
+	if ($at == $idx && $self->{pageCount} > 1) {
+		# switch away to record widget states properly
+		if ( $at > 0 ) {
+			$self->pageIndex( --$idx );
+		} else {
+			$self->pageIndex( 1 );
+			$idx = 0;
+		}
+	} elsif ( $idx > $at) {
+		$idx--;
+	}
+	$idx = 0 if $idx < 0;
 	
 	my $r = splice( @{$self-> {widgets}}, $at, 1);
 	$self-> {pageCount}--;
-	$self-> pageIndex( $self-> pageIndex);
+	$self-> {pageIndex} = $idx;
+
 	if ( $removeChildren) {
-		$_-> destroy for @$r;
+		$_-> [0]-> destroy for @$r;
 	}
 }
 
@@ -906,12 +1003,14 @@ sub pageIndex     {($#_)?($_[0]-> set_page_index   ( $_[1]))    :return $_[0]->
 sub pageCount     {($#_)?($_[0]-> set_page_count   ( $_[1]))    :return $_[0]-> {pageCount}}
 
 # TabbedNotebook styles
-package tns;
+package 
+    tns;
 use constant Simple   => 0;
 use constant Standard => 1;
 
 # TabbedNotebook orientations
-package tno;
+package 
+    tno;
 use constant Top    => 0;
 use constant Bottom => 1;
 
@@ -1048,8 +1147,33 @@ sub on_paint
 		);
 
 		my $y = $size[1] - DefBorderX + Prima::TabSet::DefGapY;
-		my $x  = $size[0] - DefBorderX - DefBookmarkX;
+		my $x = $size[0] - DefBorderX - DefBookmarkX;
 		return if $y < DefBorderX * 2 + DefBookmarkX;
+		
+		my $a  = 0;
+		my ($pi, $mpi) = ( 
+			$self-> {notebook}-> pageIndex, 
+			$self-> {notebook}-> pageCount - 1
+		);
+		$a |= 1 if $pi > 0;
+		$a |= 2 if $pi < $mpi;
+
+		if ( my $p = $self->{prelight}) {
+			$canvas-> color( $self-> prelight_color($c3d[1]));
+			if ( $p < 0 && $a & 1) {
+				$canvas->fillpoly([
+					$x - 2, $y - 2,
+					$x + DefBookmarkX - 4, $y - DefBookmarkX,
+					$x - 2, $y - DefBookmarkX,  
+				]);
+			} elsif ( $p > 0 && $a & 2 ) {
+				$canvas->fillpoly([
+					$x - 2, $y - 2,
+					$x + DefBookmarkX - 4, $y - DefBookmarkX,
+					$x + DefBookmarkX - 4, $y - 2,
+				]);
+			}
+		}
 
 		$canvas-> color( $c3d[0]);
 		$canvas-> line( 
@@ -1064,13 +1188,6 @@ sub on_paint
 		);
 
 		my $fh = 24;
-		my $a  = 0;
-		my ($pi, $mpi) = ( 
-			$self-> {notebook}-> pageIndex, 
-			$self-> {notebook}-> pageCount - 1
-		);
-		$a |= 1 if $pi > 0;
-		$a |= 2 if $pi < $mpi;
 		
 		$canvas-> line( 
 			DefBorderX + 4, $y - $fh * 1.6, 
@@ -1116,7 +1233,7 @@ sub on_paint
 			my $t1 = $$t[ $tx * 2];
 			my $yh = $y - $fh * 0.8 - $self-> font-> height / 2;
 			$canvas-> clipRect( DefBorderX + 1, $y - $fh * 1.6 + 1, $x - 4, $y - 3);
-			$canvas-> text_out( $t1, DefBorderX + 4, $yh);
+			$canvas-> text_out_bidi( $t1, DefBorderX + 4, $yh);
 			if ( $$t[ $tx * 2 + 1] > 1) {
 				$t1 = sprintf("Page %d of %d ", $self-> pageIndex - $self-> tab2page( $tx) + 1, $$t[ $tx * 2 + 1]);
 				my $tl1 = $size[0] - DefBorderX - 3 - DefBookmarkX - $self-> get_text_width( $t1);
@@ -1152,6 +1269,29 @@ sub on_mousedown
 	$self-> pageIndex( $self-> pageIndex + (( -$x + DefBookmarkX < $y) ? 1 : -1));
 }
 
+sub on_mousemove
+{
+	my ( $self, $mod, $x, $y) = @_;
+	my $prelight;
+	if (( $x, $y) = $self-> event_in_page_flipper( $x, $y)) {
+		if (-$x + DefBookmarkX < $y && $self->pageIndex < $self->pageCount - 1) {
+			$prelight = 1;
+		} elsif (-$x + DefBookmarkX >= $y && $self->pageIndex > 0 ){
+			$prelight = -1;
+		}
+	}
+	if (( $self->{prelight} // 0) != ($prelight // 0)) {
+		$self->{prelight} = $prelight;
+		$self->repaint;
+	}
+}
+
+sub on_mouseleave
+{
+	my $self = shift;
+	$self->repaint if delete $self->{prelight};
+}
+
 sub on_mousewheel
 {
 	my ( $self, $mod, $x, $y, $z) = @_;
@@ -1178,7 +1318,9 @@ sub page2tab
 	my $j = 0;
 	while( $i < $index) {
 		$j++;
-		$i += $$t[ $j*2 + 1];
+		my $n = $$t[ $j*2 + 1];
+		last unless defined $n;
+		$i += $n; 
 	}
 	return $j;
 }
@@ -1200,7 +1342,6 @@ sub TabSet_Change
 	$self-> pageIndex( $self-> tab2page( $tabset-> tabIndex));
 }
 
-
 sub set_tabs
 {
 	my $self = shift;
@@ -1335,7 +1476,71 @@ sub adjust_widgets
 	$self-> repaint;
 }
 
-sub tabIndex     {($#_)?($_[0]-> {tabSet}-> tabIndex( $_[1]))   :return $_[0]-> {tabSet}-> tabIndex}
+sub insert_page
+{
+	my ( $self, $tabName, $at ) = @_;
+
+	my $book = $self->{notebook};
+	$at = -1 unless defined $at;
+	$at = $book->pageCount + $at + 1 if $at < 0;
+	return if $at > $book->pageCount || $at < 0;
+	
+	local $self-> {changeLock} = 1;
+	$self-> {notebook}->insert_page($at);
+
+	my $ctab = $self->page2tab($at);
+	my $tabs = $self->{tabs};
+	if ( defined($tabs->[$ctab * 2]) && $tabs->[$ctab * 2] eq $tabName) {
+		$tabs->[$ctab * 2 + 1]++;
+	} elsif ( $ctab > 0 && defined($tabs->[$ctab * 2 - 2]) && $tabs->[$ctab * 2 - 2] eq $tabName) {
+		$tabs->[$ctab * 2 - 1]++;
+	} else {
+		splice( @$tabs, $ctab * 2, 0, $tabName, 1 );
+		$self-> {tabSet}->insert_tab($tabName, $ctab);
+	}
+
+	$self->repaint if $self->{style} != tns::Simple;
+
+	return $at;
+}
+
+sub delete_page
+{
+	my ( $self, $at, $removeChildren ) = @_;
+
+	my $book = $self->{notebook};
+	$at = -1 unless defined $at;
+	$at = $book->pageCount + $at if $at < 0;
+	return if $at >= $book->pageCount || $at < 0;
+
+	local $self-> {changeLock} = 1;
+	my $ctab = $self->page2tab($at);
+	my $tabs = $self->{tabs};
+
+	# stay on page within same tab, if possible
+	if ( $tabs->[$ctab * 2 + 1] > 1 && $at == $self->pageIndex && $at > 0 ) {
+		$book->pageIndex( $book->pageIndex + 1 );
+	}
+	$book->delete_page($at, $removeChildren);
+	$ctab = $self->page2tab($at);
+
+	unless ( --$tabs->[$ctab * 2 + 1] ) {
+		splice(@$tabs, $ctab * 2, 2 );
+		$self->{tabSet}->delete_tab( $ctab );
+
+		# further collapse?
+		while ( 4 < @$tabs && $ctab * 2 < @$tabs && $tabs->[$ctab * 2] eq $tabs->[$ctab * 2 - 2]) {
+			my ( undef, $n) = splice(@$tabs, $ctab * 2, 2 );
+			$tabs->[$ctab * 2 - 1] += $n;
+			$self->{tabSet}->delete_tab( $ctab );
+		}
+	}
+	$self->repaint if $self->{style} != tns::Simple;
+
+	# futher collapse
+}
+
+sub tabIndex     {($#_)?($_[0]-> {tabSet}-> tabIndex( $_[1]))  :return $_[0]-> {tabSet}-> tabIndex}
 sub pageIndex    {($#_)?($_[0]-> set_page_index   ( $_[1]))    :return $_[0]-> {notebook}-> pageIndex}
 sub tabs         {($#_)?(shift-> set_tabs     (    @_   ))     :return $_[0]-> get_tabs}
 
@@ -1715,6 +1920,14 @@ Returns width in pixels of INDEXth tab.
 Returns the index of a tab, that will be drawn leftmost if
 INDEXth tab is to be displayed.
 
+=item insert_tab TEXT, [ POSITION = -1 ]
+
+Inserts a new tab text at the given position, which is at the end by default
+
+=item delete_tab POSITION
+
+Removes a tab from the given position
+
 =back
 
 =head2 Events
@@ -1910,6 +2123,16 @@ Returns second-level tab index, that corresponds to the INDEXth first-level tab.
 Returns first-level tab index, that corresponds to the INDEXth second-level
 tab.
 
+=item insert_page TEXT, [ POSITION = -1 ]
+
+Inserts a new page with text at the given position, which is at the end by default.
+If TEXT is same as the existing tab left or right from POSITION, the page is joined
+the existing tab; otherwise a new tab is created.
+
+=item delete_page POSITION
+
+Removes a page from the given position.
+
 =back
 
 =head2 Events
